c
c------ENC_AMTTDC ------ encode AMT TDC information for drift chamber	
c
c Input:  raw(1) points 1st valid datab in the segment.
c
c Output:
c    Wire_ID: = 100*ID_plane + IW
c
c     IW       1     2            3             4
c              ID    TDC(rising)  TDC(trailing) Edge(1:L,2:T,3:LT)
c     IW       5        6           7
c              dT(L-T)  wire order  plane order
c statistics
c     ID=9991  w1(ID), w2(total word count)
c     ID=9992  w1(id), w2...w10(m_mod1, m_mod2,...,m_mod10)
c
      SUBROUTINE ENC_AMTTDC(raw,nwrd, val,nx,ny,naok)
      implicit none
c
      include 'amttdc.inc'

      integer*4 nwrd
      integer*2 raw(nwrd),iraw
      integer*4 itdc

      integer*4 nx,ny,naok
      real*4 val(nx,ny)
      include 'dc.inc'
      include 'dcmap.inc'
      integer*4 rptr
      integer*4 itag, eventid, edge
      integer*4 ip,iw,iwtot,id_tdc
      integer*4 ich, mod
c scratch
      integer*4 j,i
      integer*4 iorder
      integer*4 elen, ev_tot
      integer*4 n_ovf
      data n_ovf/0/
      save n_ovf
c
c word count
c
      if(nwrd.le.6) then ! this region contains no data but one must call 
                         ! dc multiplicity routine
         goto 01900
      endif

      if(nwrd .ge. max_evt*n_module) then
         write(0,*) 'nwrd > 259*3 longword(MAXAMTBUF), skip this data'
         goto 01900
      endif
      rptr=1
c
c vread32() in sbs620 routine transfer endian flipped data (maybe)
c sdaq routine transfer not-flipped data 
c
      do i = 1,n_module
         ev_len(i)=0
      enddo
      elen=0
      do while( rptr .le. nwrd)
         iraw=raw(rptr)
         itag = iand(iraw, (z'e000'))/(z'2000')
c         write(0,'(3Z6)') itag, iraw, raw(rptr+1)
         if( itag .eq. 5 ) then !Record data status
            elen=iand(raw(rptr),(z'1fff'))
            eventid=raw(rptr+1)
            rptr=rptr+2
         else if( itag .eq. 6 ) then ! Common time
            mod=iand(iraw, (z'1f00'))/(z'100')-m_offset ! 1:9
            ev_len(mod)=elen
c common stop time itself is ignored here.
            rptr=rptr+2
         else if( itag .eq. 2) then ! Maybe 0x5555
            rptr=rptr+2
         else if( itag .eq. 0) then ! Hit data
            edge=iand(iraw,(z'1000'))/(z'1000') ! 0 Leading, 1 Trailing
            ich=iand(iraw,(z'03f0'))/(z'0010')
            itdc=ior(iand(iraw,(z'000f'))*(z'10000'),raw(rptr+1))
            if(iand(itdc,(z'00080000')) .eq. (z'00080000')) then
c               write(0,'(I8.4,3Z12.4)') itdc,itdc, raw(rptr),raw(rptr+1)
               itdc = ior(itdc,(z'fff00000'))
c               write(0,'(I8.4,3Z12.4)') itdc,itdc, raw(rptr),raw(rptr+1)
            endif

            rptr=rptr+2
cMod ID -> 1,2,3,4,5,6,7,8,9
            iwtot=64*(mod-1)+(ich+1)
            id_tdc=itdc_to_wire(iwtot)
c//DEBUG//
c            write(0,*) (itdc_to_wire(j),j=1,16)
c            write(0,*) (itdc_to_wire(j),j=33,48)
c            write(0,*) (itdc_to_wire(j),j=81,96)
!             if(id_tdc .eq. 0) then 
!                write(*,'(a,z8,4i10)') 
!     1           'IWTOT: iraw,iwtot,mod,ich,id_tdc=',
!     1                   iraw,iwtot,mod,ich,id_tdc
!             endif
c//DEBUG//
c            write(0,'(a,6z8)') 'IWTOT :',
c     &           iraw,iwtot,mod,ich,itdc, id_tdc
         
            ip=id_tdc/100
            iw=id_tdc-100*ip
c   naok ok?
            if(naok.ge.248) then
               n_ovf=n_ovf+1
               write(*,*) 'enc_amttdc: naok >= 248,  skip the rest.',
     1                     ' nwrd, n_ovf=',nwrd,n_ovf
               go to 1900
            endif
c     fill data into buffers
            naok=naok+1
            val(1,naok)=id_tdc
            if( edge .eq. 0) then !leading edge
               val(2,naok)=itdc
               val(3,naok)=-8192
               val(4,naok)=1
               do j = naok-1,1,-1
c pair search
                  if( id_tdc .eq. val(1,j) .and. val(4,j) .eq. 2) then
                     if( val(2,j) .le. -8192 ) then ! not occupied
                        val(2,j) = itdc
                        val(4,j) = 3
                        val(5,j) = val(2,j)-val(3,j)
                        naok=naok-1
                        goto 00890
                     endif
                  endif
               enddo
c not fount 1st data
               iorder=0
               do j = naok-1,1,-1
                  if( id_tdc .eq. val(1,j) .and.
     &                 (val(4,j) .eq. 1 .or. val(4,j) .eq. 3) )then
                     iorder=iorder+1
                  endif
               enddo
               val(6,naok)=iorder
c plane order
               iorder=0
               do j = naok-1,1,-1
                  if( ip .eq. val(1,j)/100 .and.
     &                 (val(4,j) .eq. 1 .or. val(4,j) .eq. 3) )then
                     iorder=iorder+1
                  endif
               enddo
               val(7,naok)=iorder
            elseif( edge .eq. 1) then ! trailing edge
               val(2,naok)=-8192
               val(3,naok)=itdc
               val(4,naok)=2
c pair search
               do j = naok-1,1,-1
                  if( id_tdc .eq. val(1,j) .and. val(4,j) .eq. 1) then
                     if( val(3,j) .le. -8192 )then ! not occupied
                        val(3,j) = itdc
                        val(4,j) = 3
                        val(5,j) = val(2,j)-val(3,j)
                        naok=naok-1
                        goto 00890
                     endif
                  endif
               enddo
c wire order
               iorder=0
               do j = naok-1,1,-1
                  if( id_tdc .eq. val(1,j) .and.
     &                 (val(4,j) .eq. 2 .or. val(4,j) .eq. 3) )then
                     iorder=iorder+1
                  endif
               enddo
               val(6,naok)=iorder
c plane order
               iorder=0
               do j = naok-1,1,-1
                  if( ip .eq. val(1,j)/100 .and.
     &                 (val(4,j) .eq. 2 .or. val(4,j) .eq. 3) )then
                     iorder=iorder+1
                  endif
               enddo
               val(7,naok)=iorder

            endif
00890       continue
         else if( itag .eq. 3) then ! Error Report
c            write(0,'(A,I4,10Z7)')
c     &           'Data contains error report :',
c     &           rptr, (raw(rptr+j),j=-4,5)
            if(iflag_amt_error .ne. 0) then
               write(0,'(A,Z6,A,Z6,A,Z6)')
     &           ' Mod :',iand(raw(rptr),(z'1f00'))/(z'100'),
     &           ' AMT#:',iand(raw(rptr+1),(z'c000'))/(z'4000'),
     &           ' Flag:',iand(raw(rptr+1),(z'3fff'))
            endif
            rptr=rptr+2
         else
            if(iflag_amt_error .ne. 0) then
               write(0,'(A,I4,Z7,Z7)') 'Error Reading data : ',
     &              itag, raw(rptr),raw(rptr+1)
            endif
            rptr=rptr+2
         endif
      enddo
c
c   statistics
c
01900 continue
c   ID=9991
      naok=naok+1
      val(1,naok)=9991
      ev_tot=0
      do i = 1,n_module
         ev_tot= ev_tot+ev_len(i)
      enddo
      val(2,naok)=ev_tot
c   ID=9992
      naok=naok+1
      val(1,naok)= 9992
      do i = 1,n_module
         val(i+1,naok)=ev_len(i)
      enddo
c
      return
      end
c
c------ read_amttdc ------
c
      subroutine read_amttdc()
      implicit none
      include 'amttdc.inc'
      integer*4 lunin/32/
      integer*4 ier
      
      iflag_amt_error=1 ! default write
      open(lunin,file='./prm/amttdc.prm', status='old', err=00900)
      call read_int(lunin, iflag_amt_error,1,ier)
00900 continue
      return
      end
