c
c------ enc_beam ------ for He,Li(p,pN) @RIBF/Kappa Jun-2010
c                                       12-May-2010 T.Kobayashi
c
      subroutine enc_beam(raw, nwrd, val,nx,ny,naok)
c
c Input:
c   raw(1) ---> 1st valid data in segment, excluding #w_sub and eid_sub
c
c             1      2      3      4      5      6      7      8
c    TDC1:    F3L    F3R    F7L    F7R
c    TDC2:    AL     AR     BL     BR     BV     dEL    dER    dummy
c    TDC3:    EL     ER
c    ADC1:    AL     AR     BL     BR     BV     dummy
c    ADC2:    dEL    dER
c    ADC3:    EL     ER
c
c Output:
c   ID:      1      2      3      4      5      6      7      8      9
c            F3     F7     F12A   F12B   BV     dEL    dER    EL     ER
c   #PMT     2      2      2      2      1      1      1      1      1
c
c   W#:      1      2      3      4      5      6      7      8
c            ID     <T>    <A>    dT     Tl     Tr     Al     Ar
c                   12     13     14
c                   <T>_c  <A>_c  dT_c
c
      include 'beam.inc'
      include 'beam_map.inc'
      include 'ran.inc'
      integer*2 raw(1)
      dimension val(nx,ny)
      integer*4 nwrd
      dimension itdc(2)
      data iloop/0/
      save iloop
c
c read calibrations
      if(iloop.eq.0) then
          call read_beam
          iseed=12345
          iloop=1
      endif
c
      do 10 i=1,nch_b
         ihitb(i)=0
 10   continue
      iflag_t0=0
c
!      write(*,600) nwrd, (raw(k),k=1,nwrd)
! 600  format(1h ,'#words= ',i5/
!     1     1h ,10i6/1h ,10i6/1h ,10i6)              
c
c data
c
      do 100 id=1,nch_b     !loop over detectors
         ihitm=0
         do 120 i=1,2       !loop over T1,T2
            ia=mapb(i,id)   !->T1,T2
            if(ia.gt.0) then
               itdc(i)= raw(ia)
            else
               itdc(i)= -1000
            endif
            if(itdc(i).ge.itdc_low .and. 
     1         itdc(i).le.itdc_high) ihitm=ihitm+1
 120     continue
c  requirements on TDC
c     for calibration, set TDC limits to 0,10000
         if(ihitm.lt.ihitb_min(id)) go to 100  !skip this detector
         naok=naok+1
         val(1,naok)= id
         val(5,naok)= itdc(1)+(ran(iseed)-0.5)   !T1
         val(6,naok)= itdc(2)+(ran(iseed)-0.5)   !T2
         if(iflag_timcal.ne.0) then
            val(5,naok)=timcal(1,id)*val(5,naok)
            val(6,naok)=timcal(2,id)*val(6,naok)
         endif
c   ADC
         ia=mapb(3,id)  !->A1
         if(ia.gt.0) then
            val(7,naok)= raw(ia)
         else
            val(7,naok)= -1000.0
         endif
         ia= mapb(4,id)  !->A2
         if(ia.gt.0) then
            val(8,naok)= raw(ia)
         else
            val(8,naok)= -1000.0
         endif
c   pedestal
         if(iflag_ped.ne.0) then
            val(7,naok)=val(7,naok)-ped(1,id)
            val(8,naok)=val(8,naok)-ped(2,id)
!            if(val(7,naok).le.0.0 .or. val(8,naok).le.0.0) then
!               naok=naok-1   !neglect thid data
!               go to 100
!            endif
         endif
c
         if(ihitb_min(id).eq.2) then
            val(2,naok)= (val(5,naok)+val(6,naok))/2.0    !<T>
            val(4,naok)= val(5,naok)-val(6,naok)          !dT
            val(3,naok)= (val(7,naok)+val(8,naok))/2.0    !<A>
         else
            val(2,naok)= val(5,naok)
            val(4,naok)= -10000.0
            val(3,naok)= val(7,naok)
         endif
         if(iflag_t_off.ne.0) val(2,naok)=val(2,naok)-t_off(id)
         if(iflag_dt_off.ne.0) val(4,naok)=val(4,naok)-dt_off(id)
         if(iflag_gain.ne.0) 
     1        val(3,naok)=val(3,naok)*adc_norm(id)/adc_ref(id)
         ihitb(id)=naok
!c save <T_F12A> for TOF(HOD-F12A>
!         if(id.eq.1) then
!            iflag_t0=1
!            t0=val(2,naok)
!         endif
 100  continue
c
c TOF from <T_F12A>
c
!      if(ihitb(1).ne.0) then
c   F12B
!         id=2
!         if(ihitb(id).ne.0) then
!            val(15,ihitb(id))= val(2,ihitb(id))-val(2,ihitb(1))  !T(f12B)-T(f12A)
!         else
!            val(15,ihitb(id))= -1000.0
!         endif
c   F3
!         id=5
!         if(ihitb(id).ne.0) then
!            val(15,ihitb(id))= val(2,ihitb(id))-val(2,ihitb(1))  !T(f3)-T(f12A)
!         else
!            val(15,ihitb(id))= -1000.0
!         endif
c   F7
!         id=6
!         if(ihitb(id).ne.0) then
!            val(15,ihitb(id))= val(2,ihitb(id))-val(2,ihitb(1))  !T(f7)-T(f12A)
!         else
!            val(15,ihitb(id))= -1000.0
!         endif
!      else
!         val(15,ihitb(2))=-1000.0
!         val(15,ihitb(5))=-1000.0
!         val(15,ihitb(6))=-1000.0
!      endif
c
      return
      end
