c
c------ ENC_FERA ------ encode TOF & Neutron Wall
c                       mod from Be12 exp@HIMAC 18-Apr-2009  TK
c
      SUBROUTINE ENC_FERA(raw,nword, val,nx,ny,naok)
c
c INPUT: TDC & ADC are read in from CAMAC
c       no word count
c       sometimes filled with 0
c       raw(1): 1st data
c OUTPUT:
c       ID:     1:7     Hodoscope
c
c       Word#:  1       2       3       4       5       6       7       8
c               ID      <T>     <A>_cal dT      T_r     T_l     A_r     A_l
c
c               9       10      11      12      13      14      15      16
c               XfromA  sqrt(A) Z
c
      include 'fera.inc'
      include 'ran.inc'
      parameter (dist_tof=492.3)  !L(Target-HOD) [cm]
      integer*2 raw(1), idata2
      dimension val(nx,ny)
      data iloop/ 0/
      save iloop
      common/com2_beam/ iflag_t0, t0   !<T_F12A> data
c
c Parameter File
      if(iloop.eq.0) then
         call read_fera
         iloop=1
      endif
c
c initialization
c
      call icler(ihit_fera,nch_fera)
      do 70 i=1,nch_fera
         do 72 j=1,4
 72         data_fera(j,i)=-1000.0 !reset for safety
 70      continue
      mul_hod=0
c
c TFC/FERA raw data
c
!      write(*,*) '**********************************************'
!      write(*,*) 'TA ',nword,(raw(k),k=1,nword)
      NAH=1
      do 100 jadr=2,nword    !skip word count
         idata2=raw(jadr)
         if(idata2.eq.0) then   !skip zero
            go to 100
         elseif(idata2.lt.0) then !module header
            in= iand(idata2,o'377') !station#           
            if(in.le.0 .or. in.gt.23) then
               write(*,*) 'IN out of range: ',in
               go to 100
            endif
            IF(NAH.EQ.0) write(*,*) 'No data after header(A): IN_new=',in
            NAH=0
!            write(*,*) 'TDC/ADC IN=',in
         elseif(idata2.gt.0) then !data with sub_address
            NAH=NAH+1
            ia= iand(idata2,o'74000')/2048+1 !sub address 1:16
            if(ia.le.0 .or. ia.gt.16) then
               write(*,*) 'IA out of range: ',ia
               go to 100
            endif
            idata=iand(idata2,o'3777')
            id= map_fera(1,ia,in) !id=1:20
            if(id.eq.0) then
!               write(*,*) 'ID out of range: ',id
               go to 100
            endif
            iorder= map_fera(2,ia,in) !1,2,3,4 for t1,t2,a1,a2
            if(iorder.le.0 .or. iorder.gt.4) then
               write(*,*) 'IORDER out of range: ',iorder
               go to 100
            endif
!            write(*,*) 'TA: ia,idata,id,iorder=',ia,idata,id,iorder
            if(iorder.eq.1 .or. iorder.eq.2) then            !TDC
               if(idata.ge.itdc_low .and. idata.le.itdc_high) then
                  ihit_fera(id)= ihit_fera(id)+1 !hit flag
                  data_fera(iorder,id)= float(idata)+(ran(iseed)-0.5) !data
               endif
            elseif(iorder.eq.3 .or. iorder.eq.4) then        !ADC
               ihit_fera(id)= ihit_fera(id)+1
               data_fera(iorder,id)= float(idata)
            endif
         endif
 100  continue
c
c check hit flag
c
      do 300 id=1,nch_fera
!         if(ihit_fera(id).gt.0) then
!            write(*,*) 'ID, ihit_fera, ihit_min ',
!     1          id,ihit_fera(id),ihit_min(id)
!         endif
         if(ihit_fera(id).ge.ihit_min(id) .and. 
     1      ihit_min(id).gt.0) then
!            write(*,*) '------ id ------ ',
!     1           id,(data_fera(k,id),k=1,4)
            mul_hod=mul_hod+1
            naok=naok+1
            val(1,naok)= id
            do 310 i=1,4
               val(4+i,naok)= data_fera(i,id) !copy T1,t2,a1,a2
 310        continue
            val(5,naok)= val(5,naok) !T1
            val(6,naok)= val(6,naok) !T2
c       pedestal
            if(iflag_ped_fera.eq.1) then
               val(7,naok)=val(7,naok)+ped_fera(1,id)
               val(8,naok)=val(8,naok)+ped_fera(2,id)
            endif
c       gain for <A>
            tmptmp=val(7,naok)*val(8,naok)
            val(3,naok)=-1000.0
            val(12,naok)=-1000.0
            if(tmptmp.gt.0.0) then
               val(3,naok)=sqrt(tmptmp)
               if(iflag_gain_fera.eq.1 .and. adc_norm(id).gt.0.0) 
     1              val(3,naok)=val(3,naok)/gain_fera(id)*adc_norm(id)
               if(val(3,naok).gt.0.0)
     1              val(12,naok)=sqrt(val(3,naok))
            endif
c       time calibration
            if(iflag_timcal.eq.1) then
               val(5,naok)= val(5,naok)*timcal_fera(1,id)
               val(6,naok)= val(6,naok)*timcal_fera(2,id)
            endif
c       TOF average
            val(2,naok)= (val(5,naok)+val(6,naok))/2.0 !<T>
            if(iflag_t_off.eq.1) val(2,naok)=val(2,naok)-t_off_fera(id)
            val(9,naok)=-1000.0
            val(10,naok)=-1000.0
            val(11,naok)=-1000.0
            val(12,naok)=-1000.0
            if(iflag_t0.ne.0) then    !<T_F12A> available
               val(9,naok)= val(2,naok)-t0
               if(val(9,naok).gt.0.0) then
                  beta=dist_tof/30.0/val(9,naok)
                  if(beta.gt.0.0 .and. beta.lt.1.0) then
                     val(10,naok)=beta
                     val(11,naok)=val(3,naok)*beta*beta
                     if(val(11,naok).gt.0.0) then
                        tmp=val(11,naok)
                        val(11,naok)=0.3819*sqrt(tmp)
                        zr=val(11,naok)
                        val(12,naok)=0.11479+0.89301*zr-0.042132*zr**2
     1                               +0.0078496*zr**3
                     else
                        val(11,naok)= -1000.0
                     endif
                  endif
               endif
            endif
c       Time difference
            val(4,naok)= val(5,naok)-val(6,naok)   !dT
            if(iflag_dt_off.eq.1) 
     1           val(4,naok)=val(4,naok)-dt_off_fera(id)
         endif
 300  continue
c
c multiplicity
      naok=naok+1
      val(1,naok)= 99
      val(2,naok)= mul_hod
c
c exit
      return
      end
