c
c------ enc_neut ------ encode New Long Neutron wall, 11-May-2010, T.Kobayashi
c
      subroutine enc_neut(raw1,nword1, raw2,nword2, val,nx,ny,naok)
c
c Input: TDC & ADC are read from memory module via ECL transfer
c     TDC in raw1(1:nwrod1), ADC in raw2(1:nword2)
c
c VSN (2 crates into one VSN)
c   TDC   1     2     3     4     5     6      7     8      9
c         1L    1R    2L    2R    3L    3R     4L    4R     veto/joker
c
c   ADC   11    12    13    14    15    16     17    18     19
c         1L    1R    2L    2R    3L    3R     4L    4R     veto/joker
c
c Output:
c     id=  1-16, 21-36, 41-56, 61-76, 71-74, 81-88
c          N1    N2     N3     N4     Veto   Joker
c
c       Word#:  1       2       3       4       5       6       7       8
c               ID      <T>     <A>_cal dT      T_r     T_l     A_r     A_l
c
c               9       10      11      12      13      14      15      16
c               XfromA  sqrt(A) Z
c
      include 'neut.inc'
      include 'ran.inc'
      integer*2 raw1(1), raw2(1), idata2
      dimension val(nx,ny)
      dimension muln(6)
      data iloop/ 0/
      save iloop
c
c Parameter File
      if(iloop.eq.0) then
         call read_neut
         iloop=1
      endif
c
c initialization
c
      call icler(ihit_fera,nch_fera)
      do 70 i=1,nch_fera
         do 72 j=1,4
 72         data_fera(j,i)=-1000.0 !reset for safety
 70      continue
      mul_neut=0
      mul_layer=0
      do 74 j=1,6
 74   muln(j)=0    !multiplicity in one layer
c
c TFC/FERA raw data in raw1(1:nword1), raw1(1)=wc from scaler
c
!      write(*,*) '**********************************************'
!      write(*,*) 'TDC ',nword,(raw1(k),k=1,nword1)
      NAH=1
      do 100 jadr=2,nword1                 !skip word count
         idata2=raw1(jadr)
         if(idata2.eq.0) then              !skip zero
            go to 100
         elseif(idata2.lt.0) then          !module header
            if(idata2.eq.-1) go to 100     !terminator
            in= iand(idata2,o'377')        !station#           
            if(in.le.0 .or. in.gt.23) go to 100   !in out of range
            IF(NAH.EQ.0) write(*,*) 'No data after header(T): IN_new=',in
            NAH=0
!            write(*,*) 'TDC IN=',in
         elseif(idata2.gt.0) then          !data with sub_address
            NAH=NAH+1
            ia= iand(idata2,o'74000')/2048+1      !sub address 1:16
            if(ia.le.0 .or. ia.gt.16) go to 100   !ia out of range
            idata=iand(idata2,o'3777')
            id= map_fera(1,ia,in)                 !id
            if(id.eq.0) go to 100                 !id out of range
            iorder= map_fera(2,ia,in)             !1,2,3,4 for t1,t2,a1,a2
            if(iorder.le.0 .or. iorder.gt.4) go to 100 !iorder out of range
!            write(*,*) 'TDC: ia,idata,id,iorder=',ia,idata,id,iorder
            if(iorder.eq.1 .or. iorder.eq.2) then            !TDC
               if(idata.ge.itdc_low .and. idata.le.itdc_high) then
                  ihit_fera(id)= ihit_fera(id)+1             !hit flag
                  data_fera(iorder,id)= float(idata)+(ran(iseed)-0.5) !data
               endif
            endif
         endif
 100  continue
c
c ADC/FERA raw data in raw2(1:nword2), raw2(1)=wc from scaler
c
!      write(*,*) '**********************************************'
!      write(*,*) 'ADC ',nword2,(raw2(k),k=1,nword2)
      NAH=1
      do 200 jadr=2,nword2                 !skip word count
         idata2=raw2(jadr)
         if(idata2.eq.0) then              !skip zero
            go to 200
         elseif(idata2.lt.0) then          !module header
            if(idata2.eq.-1) go to 200     !terminator
            in= iand(idata2,o'377')        !station#           
            if(in.le.0 .or. in.gt.23) go to 100   !in out of range
            IF(NAH.EQ.0) write(*,*) 'No data after header(A): IN_new=',in
            NAH=0
!            write(*,*) 'ADC IN=',in
         elseif(idata2.gt.0) then          !data with sub_address
            NAH=NAH+1
            ia= iand(idata2,o'74000')/2048+1      !sub address 1:16
            if(ia.le.0 .or. ia.gt.16) go to 200   !ia out of range
            idata=iand(idata2,o'3777')
            id= map_fera(1,ia,in)                 !id
            if(id.eq.0) go to 200                 !id out of range
            iorder= map_fera(2,ia,in)             !1,2,3,4 for t1,t2,a1,a2
            if(iorder.le.0 .or. iorder.gt.4) go to 200 !iorder out of range
!            write(*,*) 'ADC: ia,idata,id,iorder=',ia,idata,id,iorder
            if(iorder.eq.3 .or. iorder.eq.4) then
               ihit_fera(id)= ihit_fera(id)+1             !hit flag
               data_fera(iorder,id)= float(idata)+(ran(iseed)-0.5) !data
            endif
         endif
 200  continue
c
c check hit flag
c
      do 300 id=1,nch_fera
!         if(ihit_fera(id).gt.0) write(*,*) 
!     1     'ID,ihit_fera,ihit_min=',id,ihit_fera(id),ihit_min(id)
         if(ihit_fera(id).ge.ihit_min(id) .and. 
     1      ihit_min(id).gt.0) then
!            write(*,*) '-id- ',id,(data_fera(k,id),k=1,4)
            mul_neut=mul_neut+1
            naok=naok+1
            val(1,naok)= id
            do 310 i=1,4
               val(4+i,naok)= data_fera(i,id) !copy T1,t2,a1,a2
 310        continue
c       pedestal
            if(iflag_ped_fera.eq.1) then
               val(7,naok)=val(7,naok)+ped_fera(1,id)
               val(8,naok)=val(8,naok)+ped_fera(2,id)
            endif
c       gain for <A>
            tmptmp=val(7,naok)*val(8,naok)
            val(3,naok)=-1000.0
            val(12,naok)=-1000.0
            if(tmptmp.gt.0.0) then
               val(3,naok)=sqrt(tmptmp)
               if(iflag_gain_fera.eq.1 .and. adc_norm(id).gt.0.0) 
     1              val(3,naok)=val(3,naok)/gain_fera(id)*adc_norm(id)
               if(val(3,naok).gt.0.0)
     1              val(12,naok)=sqrt(val(3,naok))
            endif
c       time calibration
            if(iflag_timcal.eq.1) then
               val(5,naok)= val(5,naok)*timcal_fera(1,id)
               val(6,naok)= val(6,naok)*timcal_fera(2,id)
            endif
c       TOF average
            val(2,naok)= (val(5,naok)+val(6,naok))/2.0 !<T>
            if(iflag_t_off.eq.1) val(2,naok)=val(2,naok)-t_off_fera(id)
            val(9,naok)=-1000.0
            val(10,naok)=-1000.0
            val(11,naok)=-1000.0
            val(12,naok)=-1000.0
c       Time difference
            val(4,naok)= val(5,naok)-val(6,naok)   !dT
            if(iflag_dt_off.eq.1) 
     1           val(4,naok)=val(4,naok)-dt_off_fera(id)
         endif
 300  continue
c
c check multiplicity
      if(naok.le.0) go to 499
      do 400 ip=1,6
         ip1=20*(ip-1)+1  !1,21,41,61
         ip2=ip1+15       !16,36,56,76
         if(ip.eq.5) then    !veto
            ip1=81
            ip2=84
         elseif(ip.eq.6) then  !joker
            ip1=91
            ip2=98
         endif
         do 420 jok=1,naok
            id=val(1,jok)
            if(id.ge.ip1 .and. id.le.ip2) then
               muln(ip)=muln(ip)+1
            endif
 420     continue
 400  continue
 499  continue
c
      mul_neut=0
      mul_layer=0
      do 430 ip=1,4
         mul_neut=mul_neut+muln(ip)  !multiplicity of neutron
         if(muln(ip).ge.1) mul_layer=mul_layer+1  !layer multiplicity
 430  continue
      naok=naok+1
      val(1,naok)= 101
      val(2,naok)= muln(1)  !layer1
      val(3,naok)= muln(2)  !layer2
      val(4,naok)= muln(3)  !layer3
      val(5,naok)= muln(4)  !layer4
      val(6,naok)= muln(5)  !veto
      val(7,naok)= muln(6)  !joker
      val(8,naok)= mul_neut !neutron all
      val(9,naok)= mul_layer
c
c exit
      return
      end
